@@ -18,21 +18,22 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
  ***************************************************************************/
 
 /***************************************************************************
 * STELLARIS flash is tested on LM3S811, LM3S6965, LM3s3748, more.
 ***************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include "jtag/interface.h"
 #include "imp.h"
 #include <target/algorithm.h>
 #include <target/armv7m.h>
 
-
 #define DID0_VER(did0) ((did0 >> 28)&0x07)
 
 /* STELLARIS control registers */
@@ -88,19 +89,18 @@
 #define FLASH_FMA_PRE(x)	(2 * (x))	/* for FMPPREx */
 #define FLASH_FMA_PPE(x)	(2 * (x) + 1)	/* for FMPPPEx */
 
-
 static void stellaris_read_clock_info(struct flash_bank *bank);
 static int stellaris_mass_erase(struct flash_bank *bank);
 
-struct stellaris_flash_bank
-{
+struct stellaris_flash_bank {
 	/* chip id register */
 	uint32_t did0;
 	uint32_t did1;
 	uint32_t dc0;
 	uint32_t dc1;
 
-	const char * target_name;
+	const char *target_name;
+	uint8_t target_class;
 
 	uint32_t sramsiz;
 	uint32_t flshsz;
@@ -123,194 +123,308 @@ struct stellaris_flash_bank
 	const char *mck_desc;
 };
 
-// Autogenerated by contrib/gen-stellaris-part-header.pl
-// From Stellaris Firmware Development Package revision 6734
-static struct {
-	uint32_t partno;
+/* Autogenerated by contrib/gen-stellaris-part-header.pl */
+/* From Stellaris Firmware Development Package revision 9453 */
+static const struct {
+	uint8_t class;
+	uint8_t partno;
 	const char *partname;
-}	StellarisParts[] =
-{
-	{0x0001,"LM3S101"},
-	{0x0002,"LM3S102"},
-	{0x0019,"LM3S300"},
-	{0x0011,"LM3S301"},
-	{0x001A,"LM3S308"},
-	{0x0012,"LM3S310"},
-	{0x0013,"LM3S315"},
-	{0x0014,"LM3S316"},
-	{0x0017,"LM3S317"},
-	{0x0015,"LM3S328"},
-	{0x002A,"LM3S600"},
-	{0x0021,"LM3S601"},
-	{0x002B,"LM3S608"},
-	{0x0022,"LM3S610"},
-	{0x0023,"LM3S611"},
-	{0x0024,"LM3S612"},
-	{0x0025,"LM3S613"},
-	{0x0026,"LM3S615"},
-	{0x0028,"LM3S617"},
-	{0x0029,"LM3S618"},
-	{0x0027,"LM3S628"},
-	{0x0038,"LM3S800"},
-	{0x0031,"LM3S801"},
-	{0x0039,"LM3S808"},
-	{0x0032,"LM3S811"},
-	{0x0033,"LM3S812"},
-	{0x0034,"LM3S815"},
-	{0x0036,"LM3S817"},
-	{0x0037,"LM3S818"},
-	{0x0035,"LM3S828"},
-	{0x10BF,"LM3S1110"},
-	{0x10C3,"LM3S1133"},
-	{0x10C5,"LM3S1138"},
-	{0x10C1,"LM3S1150"},
-	{0x10C4,"LM3S1162"},
-	{0x10C2,"LM3S1165"},
-	{0x10C6,"LM3S1332"},
-	{0x10BC,"LM3S1435"},
-	{0x10BA,"LM3S1439"},
-	{0x10BB,"LM3S1512"},
-	{0x10C7,"LM3S1538"},
-	{0x10DB,"LM3S1601"},
-	{0x1006,"LM3S1607"},
-	{0x10DA,"LM3S1608"},
-	{0x10C0,"LM3S1620"},
-	{0x1003,"LM3S1625"},
-	{0x1004,"LM3S1626"},
-	{0x1005,"LM3S1627"},
-	{0x10B3,"LM3S1635"},
-	{0x10BD,"LM3S1637"},
-	{0x10B1,"LM3S1651"},
-	{0x10B9,"LM3S1751"},
-	{0x1010,"LM3S1776"},
-	{0x1016,"LM3S1811"},
-	{0x103D,"LM3S1816"},
-	{0x10B4,"LM3S1850"},
-	{0x10DD,"LM3S1911"},
-	{0x10DC,"LM3S1918"},
-	{0x10B7,"LM3S1937"},
-	{0x10BE,"LM3S1958"},
-	{0x10B5,"LM3S1960"},
-	{0x10B8,"LM3S1968"},
-	{0x100F,"LM3S1J11"},
-	{0x103C,"LM3S1J16"},
-	{0x100E,"LM3S1N11"},
-	{0x103B,"LM3S1N16"},
-	{0x10B2,"LM3S1P51"},
-	{0x109E,"LM3S1R21"},
-	{0x1030,"LM3S1W16"},
-	{0x102F,"LM3S1Z16"},
-	{0x1051,"LM3S2110"},
-	{0x1084,"LM3S2139"},
-	{0x1039,"LM3S2276"},
-	{0x10A2,"LM3S2410"},
-	{0x1059,"LM3S2412"},
-	{0x1056,"LM3S2432"},
-	{0x105A,"LM3S2533"},
-	{0x10E1,"LM3S2601"},
-	{0x10E0,"LM3S2608"},
-	{0x1033,"LM3S2616"},
-	{0x1057,"LM3S2620"},
-	{0x1085,"LM3S2637"},
-	{0x1053,"LM3S2651"},
-	{0x1080,"LM3S2671"},
-	{0x1050,"LM3S2678"},
-	{0x10A4,"LM3S2730"},
-	{0x1052,"LM3S2739"},
-	{0x103A,"LM3S2776"},
-	{0x106D,"LM3S2793"},
-	{0x10E3,"LM3S2911"},
-	{0x10E2,"LM3S2918"},
-	{0x1054,"LM3S2939"},
-	{0x108F,"LM3S2948"},
-	{0x1058,"LM3S2950"},
-	{0x1055,"LM3S2965"},
-	{0x106C,"LM3S2B93"},
-	{0x1008,"LM3S3634"},
-	{0x1043,"LM3S3651"},
-	{0x1044,"LM3S3739"},
-	{0x1049,"LM3S3748"},
-	{0x1045,"LM3S3749"},
-	{0x1042,"LM3S3826"},
-	{0x1041,"LM3S3J26"},
-	{0x1040,"LM3S3N26"},
-	{0x103F,"LM3S3W26"},
-	{0x103E,"LM3S3Z26"},
-	{0x1081,"LM3S5632"},
-	{0x100C,"LM3S5651"},
-	{0x108A,"LM3S5652"},
-	{0x104D,"LM3S5656"},
-	{0x1091,"LM3S5662"},
-	{0x1096,"LM3S5732"},
-	{0x1097,"LM3S5737"},
-	{0x10A0,"LM3S5739"},
-	{0x1099,"LM3S5747"},
-	{0x10A7,"LM3S5749"},
-	{0x109A,"LM3S5752"},
-	{0x109C,"LM3S5762"},
-	{0x1069,"LM3S5791"},
-	{0x100B,"LM3S5951"},
-	{0x104E,"LM3S5956"},
-	{0x1068,"LM3S5B91"},
-	{0x1009,"LM3S5K31"},
-	{0x104A,"LM3S5K36"},
-	{0x100A,"LM3S5P31"},
-	{0x1048,"LM3S5P36"},
-	{0x100D,"LM3S5P51"},
-	{0x104C,"LM3S5P56"},
-	{0x1007,"LM3S5R31"},
-	{0x104B,"LM3S5R36"},
-	{0x1047,"LM3S5T36"},
-	{0x1046,"LM3S5Y36"},
-	{0x10A1,"LM3S6100"},
-	{0x1074,"LM3S6110"},
-	{0x10A5,"LM3S6420"},
-	{0x1082,"LM3S6422"},
-	{0x1075,"LM3S6432"},
-	{0x1076,"LM3S6537"},
-	{0x1071,"LM3S6610"},
-	{0x10E7,"LM3S6611"},
-	{0x10E6,"LM3S6618"},
-	{0x1083,"LM3S6633"},
-	{0x108B,"LM3S6637"},
-	{0x10A3,"LM3S6730"},
-	{0x1077,"LM3S6753"},
-	{0x10E9,"LM3S6911"},
-	{0x10E8,"LM3S6918"},
-	{0x1089,"LM3S6938"},
-	{0x1072,"LM3S6950"},
-	{0x1078,"LM3S6952"},
-	{0x1073,"LM3S6965"},
-	{0x1064,"LM3S8530"},
-	{0x108E,"LM3S8538"},
-	{0x1061,"LM3S8630"},
-	{0x1063,"LM3S8730"},
-	{0x108D,"LM3S8733"},
-	{0x1086,"LM3S8738"},
-	{0x1065,"LM3S8930"},
-	{0x108C,"LM3S8933"},
-	{0x1088,"LM3S8938"},
-	{0x10A6,"LM3S8962"},
-	{0x1062,"LM3S8970"},
-	{0x10D7,"LM3S8971"},
-	{0x1067,"LM3S9790"},
-	{0x106B,"LM3S9792"},
-	{0x1020,"LM3S9997"},
-	{0x1066,"LM3S9B90"},
-	{0x106A,"LM3S9B92"},
-	{0x106E,"LM3S9B95"},
-	{0x106F,"LM3S9B96"},
-	{0x1018,"LM3S9L97"},
-	{0,"Unknown part"}
+} StellarisParts[] = {
+	{0x00, 0x01, "LM3S101"},
+	{0x00, 0x02, "LM3S102"},
+	{0x01, 0xBF, "LM3S1110"},
+	{0x01, 0xC3, "LM3S1133"},
+	{0x01, 0xC5, "LM3S1138"},
+	{0x01, 0xC1, "LM3S1150"},
+	{0x01, 0xC4, "LM3S1162"},
+	{0x01, 0xC2, "LM3S1165"},
+	{0x01, 0xEC, "LM3S1166"},
+	{0x01, 0xC6, "LM3S1332"},
+	{0x01, 0xBC, "LM3S1435"},
+	{0x01, 0xBA, "LM3S1439"},
+	{0x01, 0xBB, "LM3S1512"},
+	{0x01, 0xC7, "LM3S1538"},
+	{0x01, 0xDB, "LM3S1601"},
+	{0x03, 0x06, "LM3S1607"},
+	{0x01, 0xDA, "LM3S1608"},
+	{0x01, 0xC0, "LM3S1620"},
+	{0x04, 0xCD, "LM3S1621"},
+	{0x03, 0x03, "LM3S1625"},
+	{0x03, 0x04, "LM3S1626"},
+	{0x03, 0x05, "LM3S1627"},
+	{0x01, 0xB3, "LM3S1635"},
+	{0x01, 0xEB, "LM3S1636"},
+	{0x01, 0xBD, "LM3S1637"},
+	{0x04, 0xB1, "LM3S1651"},
+	{0x01, 0xB9, "LM3S1751"},
+	{0x03, 0x10, "LM3S1776"},
+	{0x04, 0x16, "LM3S1811"},
+	{0x04, 0x3D, "LM3S1816"},
+	{0x01, 0xB4, "LM3S1850"},
+	{0x01, 0xDD, "LM3S1911"},
+	{0x01, 0xDC, "LM3S1918"},
+	{0x01, 0xB7, "LM3S1937"},
+	{0x01, 0xBE, "LM3S1958"},
+	{0x01, 0xB5, "LM3S1960"},
+	{0x01, 0xB8, "LM3S1968"},
+	{0x01, 0xEA, "LM3S1969"},
+	{0x04, 0xCE, "LM3S1B21"},
+	{0x06, 0xCA, "LM3S1C21"},
+	{0x06, 0xCB, "LM3S1C26"},
+	{0x06, 0x98, "LM3S1C58"},
+	{0x06, 0xB0, "LM3S1D21"},
+	{0x06, 0xCC, "LM3S1D26"},
+	{0x06, 0x1D, "LM3S1F11"},
+	{0x06, 0x1B, "LM3S1F16"},
+	{0x06, 0xAF, "LM3S1G21"},
+	{0x06, 0x95, "LM3S1G58"},
+	{0x06, 0x1E, "LM3S1H11"},
+	{0x06, 0x1C, "LM3S1H16"},
+	{0x04, 0x0F, "LM3S1J11"},
+	{0x04, 0x3C, "LM3S1J16"},
+	{0x04, 0x0E, "LM3S1N11"},
+	{0x04, 0x3B, "LM3S1N16"},
+	{0x04, 0xB2, "LM3S1P51"},
+	{0x04, 0x9E, "LM3S1R21"},
+	{0x04, 0xC9, "LM3S1R26"},
+	{0x04, 0x30, "LM3S1W16"},
+	{0x04, 0x2F, "LM3S1Z16"},
+	{0x01, 0x51, "LM3S2110"},
+	{0x01, 0x84, "LM3S2139"},
+	{0x03, 0x39, "LM3S2276"},
+	{0x01, 0xA2, "LM3S2410"},
+	{0x01, 0x59, "LM3S2412"},
+	{0x01, 0x56, "LM3S2432"},
+	{0x01, 0x5A, "LM3S2533"},
+	{0x01, 0xE1, "LM3S2601"},
+	{0x01, 0xE0, "LM3S2608"},
+	{0x03, 0x33, "LM3S2616"},
+	{0x01, 0x57, "LM3S2620"},
+	{0x01, 0x85, "LM3S2637"},
+	{0x01, 0x53, "LM3S2651"},
+	{0x03, 0x80, "LM3S2671"},
+	{0x03, 0x50, "LM3S2678"},
+	{0x01, 0xA4, "LM3S2730"},
+	{0x01, 0x52, "LM3S2739"},
+	{0x03, 0x3A, "LM3S2776"},
+	{0x04, 0x6D, "LM3S2793"},
+	{0x01, 0xE3, "LM3S2911"},
+	{0x01, 0xE2, "LM3S2918"},
+	{0x01, 0xED, "LM3S2919"},
+	{0x01, 0x54, "LM3S2939"},
+	{0x01, 0x8F, "LM3S2948"},
+	{0x01, 0x58, "LM3S2950"},
+	{0x01, 0x55, "LM3S2965"},
+	{0x04, 0x6C, "LM3S2B93"},
+	{0x06, 0x94, "LM3S2D93"},
+	{0x06, 0x93, "LM3S2U93"},
+	{0x00, 0x19, "LM3S300"},
+	{0x00, 0x11, "LM3S301"},
+	{0x00, 0x1A, "LM3S308"},
+	{0x00, 0x12, "LM3S310"},
+	{0x00, 0x13, "LM3S315"},
+	{0x00, 0x14, "LM3S316"},
+	{0x00, 0x17, "LM3S317"},
+	{0x00, 0x15, "LM3S328"},
+	{0x03, 0x08, "LM3S3634"},
+	{0x03, 0x43, "LM3S3651"},
+	{0x04, 0xC8, "LM3S3654"},
+	{0x03, 0x44, "LM3S3739"},
+	{0x03, 0x49, "LM3S3748"},
+	{0x03, 0x45, "LM3S3749"},
+	{0x04, 0x42, "LM3S3826"},
+	{0x04, 0x41, "LM3S3J26"},
+	{0x04, 0x40, "LM3S3N26"},
+	{0x04, 0x3F, "LM3S3W26"},
+	{0x04, 0x3E, "LM3S3Z26"},
+	{0x03, 0x81, "LM3S5632"},
+	{0x04, 0x0C, "LM3S5651"},
+	{0x03, 0x8A, "LM3S5652"},
+	{0x04, 0x4D, "LM3S5656"},
+	{0x03, 0x91, "LM3S5662"},
+	{0x03, 0x96, "LM3S5732"},
+	{0x03, 0x97, "LM3S5737"},
+	{0x03, 0xA0, "LM3S5739"},
+	{0x03, 0x99, "LM3S5747"},
+	{0x03, 0xA7, "LM3S5749"},
+	{0x03, 0x9A, "LM3S5752"},
+	{0x03, 0x9C, "LM3S5762"},
+	{0x04, 0x69, "LM3S5791"},
+	{0x04, 0x0B, "LM3S5951"},
+	{0x04, 0x4E, "LM3S5956"},
+	{0x04, 0x68, "LM3S5B91"},
+	{0x06, 0x2E, "LM3S5C31"},
+	{0x06, 0x2C, "LM3S5C36"},
+	{0x06, 0x5E, "LM3S5C51"},
+	{0x06, 0x5B, "LM3S5C56"},
+	{0x06, 0x5F, "LM3S5D51"},
+	{0x06, 0x5C, "LM3S5D56"},
+	{0x06, 0x87, "LM3S5D91"},
+	{0x06, 0x2D, "LM3S5G31"},
+	{0x06, 0x1F, "LM3S5G36"},
+	{0x06, 0x5D, "LM3S5G51"},
+	{0x06, 0x4F, "LM3S5G56"},
+	{0x04, 0x09, "LM3S5K31"},
+	{0x04, 0x4A, "LM3S5K36"},
+	{0x04, 0x0A, "LM3S5P31"},
+	{0x04, 0x48, "LM3S5P36"},
+	{0x04, 0xB6, "LM3S5P3B"},
+	{0x04, 0x0D, "LM3S5P51"},
+	{0x04, 0x4C, "LM3S5P56"},
+	{0x04, 0x07, "LM3S5R31"},
+	{0x04, 0x4B, "LM3S5R36"},
+	{0x04, 0x47, "LM3S5T36"},
+	{0x06, 0x7F, "LM3S5U91"},
+	{0x04, 0x46, "LM3S5Y36"},
+	{0x00, 0x2A, "LM3S600"},
+	{0x00, 0x21, "LM3S601"},
+	{0x00, 0x2B, "LM3S608"},
+	{0x00, 0x22, "LM3S610"},
+	{0x01, 0xA1, "LM3S6100"},
+	{0x00, 0x23, "LM3S611"},
+	{0x01, 0x74, "LM3S6110"},
+	{0x00, 0x24, "LM3S612"},
+	{0x00, 0x25, "LM3S613"},
+	{0x00, 0x26, "LM3S615"},
+	{0x00, 0x28, "LM3S617"},
+	{0x00, 0x29, "LM3S618"},
+	{0x00, 0x27, "LM3S628"},
+	{0x01, 0xA5, "LM3S6420"},
+	{0x01, 0x82, "LM3S6422"},
+	{0x01, 0x75, "LM3S6432"},
+	{0x01, 0x76, "LM3S6537"},
+	{0x01, 0x71, "LM3S6610"},
+	{0x01, 0xE7, "LM3S6611"},
+	{0x01, 0xE6, "LM3S6618"},
+	{0x01, 0x83, "LM3S6633"},
+	{0x01, 0x8B, "LM3S6637"},
+	{0x01, 0xA3, "LM3S6730"},
+	{0x01, 0x77, "LM3S6753"},
+	{0x01, 0xE9, "LM3S6911"},
+	{0x01, 0xE8, "LM3S6918"},
+	{0x01, 0x89, "LM3S6938"},
+	{0x01, 0x72, "LM3S6950"},
+	{0x01, 0x78, "LM3S6952"},
+	{0x01, 0x73, "LM3S6965"},
+	{0x06, 0xAA, "LM3S6C11"},
+	{0x06, 0xAC, "LM3S6C65"},
+	{0x06, 0x9F, "LM3S6G11"},
+	{0x06, 0xAB, "LM3S6G65"},
+	{0x00, 0x38, "LM3S800"},
+	{0x00, 0x31, "LM3S801"},
+	{0x00, 0x39, "LM3S808"},
+	{0x00, 0x32, "LM3S811"},
+	{0x00, 0x33, "LM3S812"},
+	{0x00, 0x34, "LM3S815"},
+	{0x00, 0x36, "LM3S817"},
+	{0x00, 0x37, "LM3S818"},
+	{0x00, 0x35, "LM3S828"},
+	{0x01, 0x64, "LM3S8530"},
+	{0x01, 0x8E, "LM3S8538"},
+	{0x01, 0x61, "LM3S8630"},
+	{0x01, 0x63, "LM3S8730"},
+	{0x01, 0x8D, "LM3S8733"},
+	{0x01, 0x86, "LM3S8738"},
+	{0x01, 0x65, "LM3S8930"},
+	{0x01, 0x8C, "LM3S8933"},
+	{0x01, 0x88, "LM3S8938"},
+	{0x01, 0xA6, "LM3S8962"},
+	{0x01, 0x62, "LM3S8970"},
+	{0x01, 0xD7, "LM3S8971"},
+	{0x06, 0xAE, "LM3S8C62"},
+	{0x06, 0xAD, "LM3S8G62"},
+	{0x04, 0xCF, "LM3S9781"},
+	{0x04, 0x67, "LM3S9790"},
+	{0x04, 0x6B, "LM3S9792"},
+	{0x04, 0x2D, "LM3S9971"},
+	{0x04, 0x20, "LM3S9997"},
+	{0x04, 0xD0, "LM3S9B81"},
+	{0x04, 0x66, "LM3S9B90"},
+	{0x04, 0x6A, "LM3S9B92"},
+	{0x04, 0x6E, "LM3S9B95"},
+	{0x04, 0x6F, "LM3S9B96"},
+	{0x04, 0x1D, "LM3S9BN2"},
+	{0x04, 0x1E, "LM3S9BN5"},
+	{0x04, 0x1F, "LM3S9BN6"},
+	{0x06, 0x70, "LM3S9C97"},
+	{0x06, 0xA9, "LM3S9D81"},
+	{0x06, 0x7E, "LM3S9D90"},
+	{0x06, 0x92, "LM3S9D92"},
+	{0x06, 0x9D, "LM3S9D96"},
+	{0x06, 0x7B, "LM3S9DN5"},
+	{0x06, 0x7C, "LM3S9DN6"},
+	{0x06, 0x60, "LM3S9G97"},
+	{0x06, 0x79, "LM3S9GN5"},
+	{0x04, 0x1B, "LM3S9L71"},
+	{0x04, 0x18, "LM3S9L97"},
+	{0x06, 0xA8, "LM3S9U81"},
+	{0x06, 0x7D, "LM3S9U90"},
+	{0x06, 0x90, "LM3S9U92"},
+	{0x06, 0x9B, "LM3S9U96"},
+	{0x05, 0x18, "LM4F110B2QR"},
+	{0x05, 0x19, "LM4F110C4QR"},
+	{0x05, 0x10, "LM4F110E5QR"},
+	{0x05, 0x11, "LM4F110H5QR"},
+	{0x05, 0x22, "LM4F111B2QR"},
+	{0x05, 0x23, "LM4F111C4QR"},
+	{0x05, 0x20, "LM4F111E5QR"},
+	{0x05, 0x21, "LM4F111H5QR"},
+	{0x05, 0x36, "LM4F112C4QC"},
+	{0x05, 0x30, "LM4F112E5QC"},
+	{0x05, 0x31, "LM4F112H5QC"},
+	{0x05, 0x35, "LM4F112H5QD"},
+	{0x05, 0x01, "LM4F120B2QR"},
+	{0x05, 0x02, "LM4F120C4QR"},
+	{0x05, 0x03, "LM4F120E5QR"},
+	{0x05, 0x04, "LM4F120H5QR"},
+	{0x05, 0x08, "LM4F121B2QR"},
+	{0x05, 0x09, "LM4F121C4QR"},
+	{0x05, 0x0A, "LM4F121E5QR"},
+	{0x05, 0x0B, "LM4F121H5QR"},
+	{0x05, 0xD0, "LM4F122C4QC"},
+	{0x05, 0xD1, "LM4F122E5QC"},
+	{0x05, 0xD2, "LM4F122H5QC"},
+	{0x05, 0xD6, "LM4F122H5QD"},
+	{0x05, 0x48, "LM4F130C4QR"},
+	{0x05, 0x40, "LM4F130E5QR"},
+	{0x05, 0x41, "LM4F130H5QR"},
+	{0x05, 0x52, "LM4F131C4QR"},
+	{0x05, 0x50, "LM4F131E5QR"},
+	{0x05, 0x51, "LM4F131H5QR"},
+	{0x05, 0x66, "LM4F132C4QC"},
+	{0x05, 0x60, "LM4F132E5QC"},
+	{0x05, 0x61, "LM4F132H5QC"},
+	{0x05, 0x65, "LM4F132H5QD"},
+	{0x05, 0x70, "LM4F210E5QR"},
+	{0x05, 0x73, "LM4F210H5QR"},
+	{0x05, 0x80, "LM4F211E5QR"},
+	{0x05, 0x83, "LM4F211H5QR"},
+	{0x05, 0xE9, "LM4F212H5BB"},
+	{0x05, 0xC4, "LM4F212H5QC"},
+	{0x05, 0xC6, "LM4F212H5QD"},
+	{0x05, 0xA0, "LM4F230E5QR"},
+	{0x05, 0xA1, "LM4F230H5QR"},
+	{0x05, 0xB0, "LM4F231E5QR"},
+	{0x05, 0xB1, "LM4F231H5QR"},
+	{0x05, 0xC0, "LM4F232E5QC"},
+	{0x05, 0xE3, "LM4F232H5BB"},
+	{0x05, 0xC1, "LM4F232H5QC"},
+	{0x05, 0xC5, "LM4F232H5QD"},
+	{0x05, 0xE5, "LM4FS1AH5BB"},
+	{0x05, 0xEA, "LM4FS1GH5BB"},
+	{0x05, 0xE4, "LM4FS99H5BB"},
+	{0x05, 0xE1, "LM4FSXLH5BB"},
+	{0xFF, 0x00, "Unknown Part"}
 };
 
-static char * StellarisClassname[5] =
-{
+static const char *StellarisClassname[7] = {
 	"Sandstorm",
 	"Fury",
 	"Unknown",
 	"DustDevil",
-	"Tempest"
+	"Tempest",
+	"Blizzard",
+	"Firestorm"
 };
 
 /***************************************************************************
@@ -324,10 +438,7 @@ FLASH_BANK_COMMAND_HANDLER(stellaris_flash_bank_command)
 	struct stellaris_flash_bank *stellaris_info;
 
 	if (CMD_ARGC < 6)
-	{
-		LOG_WARNING("incomplete flash_bank stellaris configuration");
-		return ERROR_FLASH_BANK_INVALID;
-	}
+		return ERROR_COMMAND_SYNTAX_ERROR;
 
 	stellaris_info = calloc(sizeof(struct stellaris_flash_bank), 1);
 	bank->base = 0x0;
@@ -347,7 +458,7 @@ FLASH_BANK_COMMAND_HANDLER(stellaris_flash_bank_command)
 
 static int get_stellaris_info(struct flash_bank *bank, char *buf, int buf_size)
 {
-	int printed, device_class;
+	int printed;
 	struct stellaris_flash_bank *stellaris_info = bank->driver_priv;
 
 	if (stellaris_info->did1 == 0)
@@ -356,20 +467,12 @@ static int get_stellaris_info(struct flash_bank *bank, char *buf, int buf_size)
 	/* Read main and master clock freqency register */
 	stellaris_read_clock_info(bank);
 
-	if (DID0_VER(stellaris_info->did0) > 0)
-	{
-		device_class = (stellaris_info->did0 >> 16) & 0xFF;
-	}
-	else
-	{
-		device_class = 0;
-	}
 	printed = snprintf(buf,
 			   buf_size,
 			   "\nTI/LMI Stellaris information: Chip is "
 			   "class %i (%s) %s rev %c%i\n",
-			   device_class,
-			   StellarisClassname[device_class],
+			   stellaris_info->target_class,
+			   StellarisClassname[stellaris_info->target_class],
 			   stellaris_info->target_name,
 			   (int)('A' + ((stellaris_info->did0 >> 8) & 0xFF)),
 			   (int)((stellaris_info->did0) & 0xFF));
@@ -399,9 +502,8 @@ static int get_stellaris_info(struct flash_bank *bank, char *buf, int buf_size)
 	buf += printed;
 	buf_size -= printed;
 
-	if (stellaris_info->num_lockbits > 0)
-	{
-		printed = snprintf(buf,
+	if (stellaris_info->num_lockbits > 0) {
+		snprintf(buf,
 				buf_size,
 				"pagesize: %" PRIi32 ", pages: %d, "
 				"lockbits: %i, pages per lockbit: %i\n",
@@ -409,8 +511,6 @@ static int get_stellaris_info(struct flash_bank *bank, char *buf, int buf_size)
 				(unsigned) stellaris_info->num_pages,
 				stellaris_info->num_lockbits,
 				(unsigned) stellaris_info->pages_in_lockregion);
-		buf += printed;
-		buf_size -= printed;
 	}
 	return ERROR_OK;
 }
@@ -426,7 +526,11 @@ static void stellaris_set_flash_timing(struct flash_bank *bank)
 	struct target *target = bank->target;
 	uint32_t usecrl = (stellaris_info->mck_freq/1000000ul-1);
 
-	LOG_DEBUG("usecrl = %i",(int)(usecrl));
+	/* only valid for Sandstorm and Fury class devices */
+	if (stellaris_info->target_class > 1)
+		return;
+
+	LOG_DEBUG("usecrl = %i", (int)(usecrl));
 	target_write_u32(target, SCB_BASE | USECRL, usecrl);
 }
 
@@ -507,8 +611,7 @@ static void stellaris_read_clock_info(struct flash_bank *bank)
 
 	stellaris_info->mck_desc = "";
 
-	switch (oscsrc)
-	{
+	switch (oscsrc) {
 		case 0:				/* MOSC */
 			mainfreq = rcc_xtal[xtal];
 			break;
@@ -564,22 +667,19 @@ static int stellaris_read_part_info(struct flash_bank *bank)
 		  did0, did1, stellaris_info->dc0, stellaris_info->dc1);
 
 	ver = did0 >> 28;
-	if ((ver != 0) && (ver != 1))
-	{
+	if ((ver != 0) && (ver != 1)) {
 		LOG_WARNING("Unknown did0 version, cannot identify target");
 		return ERROR_FLASH_OPERATION_FAILED;
 	}
 
-	if (did1 == 0)
-	{
+	if (did1 == 0) {
 		LOG_WARNING("Cannot identify target as a Stellaris");
 		return ERROR_FLASH_OPERATION_FAILED;
 	}
 
 	ver = did1 >> 28;
 	fam = (did1 >> 24) & 0xF;
-	if (((ver != 0) && (ver != 1)) || (fam != 0))
-	{
+	if (((ver != 0) && (ver != 1)) || (fam != 0)) {
 		LOG_WARNING("Unknown did1 version/family.");
 		return ERROR_FLASH_OPERATION_FAILED;
 	}
@@ -590,46 +690,55 @@ static int stellaris_read_part_info(struct flash_bank *bank)
 	 * always approximate.
 	 *
 	 * For Tempest:  IOSC is calibrated, 16 MHz
+	 * For Blizzard:  IOSC is calibrated, 16 MHz
+	 * For Firestorm:  IOSC is calibrated, 16 MHz
 	 */
 	stellaris_info->iosc_freq = 12000000;
 	stellaris_info->iosc_desc = " (±30%)";
 	stellaris_info->xtal_mask = 0x0f;
 
-	switch ((did0 >> 28) & 0x7) {
-	case 0:				/* Sandstorm */
-		/*
-		 * Current (2009-August) parts seem to be rev C2 and use 12 MHz.
-		 * Parts before rev C0 used 15 MHz; some C0 parts use 15 MHz
-		 * (LM3S618), but some other C0 parts are 12 MHz (LM3S811).
-		 */
-		if (((did0 >> 8) & 0xff) < 2) {
-			stellaris_info->iosc_freq = 15000000;
-			stellaris_info->iosc_desc = " (±50%)";
-		}
-		break;
-	case 1:
-		switch ((did0 >> 16) & 0xff) {
+	/* get device class */
+	if (DID0_VER(did0) > 0) {
+		stellaris_info->target_class = (did0 >> 16) & 0xFF;
+	} else {
+		/* Sandstorm class */
+		stellaris_info->target_class = 0;
+	}
+
+	switch (stellaris_info->target_class) {
+		case 0:				/* Sandstorm */
+			/*
+			 * Current (2009-August) parts seem to be rev C2 and use 12 MHz.
+			 * Parts before rev C0 used 15 MHz; some C0 parts use 15 MHz
+			 * (LM3S618), but some other C0 parts are 12 MHz (LM3S811).
+			 */
+			if (((did0 >> 8) & 0xff) < 2) {
+				stellaris_info->iosc_freq = 15000000;
+				stellaris_info->iosc_desc = " (±50%)";
+			}
+			break;
+
 		case 1:			/* Fury */
 			break;
+
 		case 4:			/* Tempest */
+		case 5:			/* Blizzard */
+		case 6:			/* Firestorm */
 			stellaris_info->iosc_freq = 16000000;	/* +/- 1% */
 			stellaris_info->iosc_desc = " (±1%)";
 			/* FALL THROUGH */
+
 		case 3:			/* DustDevil */
 			stellaris_info->xtal_mask = 0x1f;
 			break;
+
 		default:
 			LOG_WARNING("Unknown did0 class");
-		}
-		break;
-	default:
-		LOG_WARNING("Unknown did0 version");
-		break;
 	}
 
-	for (i = 0; StellarisParts[i].partno; i++)
-	{
-		if (StellarisParts[i].partno == ((did1 >> 16) & 0xFFFF))
+	for (i = 0; StellarisParts[i].partno; i++) {
+		if ((StellarisParts[i].partno == ((did1 >> 16) & 0xFF)) &&
+				(StellarisParts[i].class == stellaris_info->target_class))
 			break;
 	}
 
@@ -639,7 +748,7 @@ static int stellaris_read_part_info(struct flash_bank *bank)
 	stellaris_info->did1 = did1;
 
 	stellaris_info->num_lockbits = 1 + (stellaris_info->dc0 & 0xFFFF);
-	stellaris_info->num_pages = 2 *(1 + (stellaris_info->dc0 & 0xFFFF));
+	stellaris_info->num_pages = 2 * (1 + (stellaris_info->dc0 & 0xFFFF));
 	stellaris_info->pagesize = 1024;
 	stellaris_info->pages_in_lockregion = 2;
 
@@ -708,8 +817,7 @@ static int stellaris_erase(struct flash_bank *bank, int first, int last)
 	struct stellaris_flash_bank *stellaris_info = bank->driver_priv;
 	struct target *target = bank->target;
 
-	if (bank->target->state != TARGET_HALTED)
-	{
+	if (bank->target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
@@ -718,14 +826,10 @@ static int stellaris_erase(struct flash_bank *bank, int first, int last)
 		return ERROR_FLASH_BANK_NOT_PROBED;
 
 	if ((first < 0) || (last < first) || (last >= (int)stellaris_info->num_pages))
-	{
 		return ERROR_FLASH_SECTOR_INVALID;
-	}
 
 	if ((first == 0) && (last == ((int)stellaris_info->num_pages-1)))
-	{
 		return stellaris_mass_erase(bank);
-	}
 
 	/* Refresh flash controller timing */
 	stellaris_read_clock_info(bank);
@@ -739,24 +843,21 @@ static int stellaris_erase(struct flash_bank *bank, int first, int last)
 	 * it might want to process those IRQs.
 	 */
 
-	for (banknr = first; banknr <= last; banknr++)
-	{
+	for (banknr = first; banknr <= last; banknr++) {
 		/* Address is first word in page */
 		target_write_u32(target, FLASH_FMA, banknr * stellaris_info->pagesize);
 		/* Write erase command */
 		target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_ERASE);
 		/* Wait until erase complete */
-		do
-		{
+		do {
 			target_read_u32(target, FLASH_FMC, &flash_fmc);
-		}
-		while (flash_fmc & FMC_ERASE);
+		} while (flash_fmc & FMC_ERASE);
 
 		/* Check acess violations */
 		target_read_u32(target, FLASH_CRIS, &flash_cris);
-		if (flash_cris & (AMASK))
-		{
-			LOG_WARNING("Error erasing flash page %i,  flash_cris 0x%" PRIx32 "", banknr, flash_cris);
+		if (flash_cris & (AMASK)) {
+			LOG_WARNING("Error erasing flash page %i,  flash_cris 0x%" PRIx32 "",
+					banknr, flash_cris);
 			target_write_u32(target, FLASH_CRIS, 0);
 			return ERROR_FLASH_OPERATION_FAILED;
 		}
@@ -775,17 +876,15 @@ static int stellaris_protect(struct flash_bank *bank, int set, int first, int la
 	struct stellaris_flash_bank *stellaris_info = bank->driver_priv;
 	struct target *target = bank->target;
 
-	if (bank->target->state != TARGET_HALTED)
-	{
+	if (bank->target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
 
-	if (!set)
-	{
+	if (!set) {
 		LOG_ERROR("Hardware doesn't support page-level unprotect. "
 			"Try the 'recover' command.");
-		return ERROR_INVALID_ARGUMENTS;
+		return ERROR_COMMAND_SYNTAX_ERROR;
 	}
 
 	if (stellaris_info->did1 == 0)
@@ -794,8 +893,7 @@ static int stellaris_protect(struct flash_bank *bank, int set, int first, int la
 	/* lockregions are 2 pages ... must protect [even..odd] */
 	if ((first < 0) || (first & 1)
 			|| (last < first) || !(last & 1)
-			|| (last >= 2 * stellaris_info->num_lockbits))
-	{
+			|| (last >= 2 * stellaris_info->num_lockbits)) {
 		LOG_ERROR("Can't protect unaligned or out-of-range pages.");
 		return ERROR_FLASH_SECTOR_INVALID;
 	}
@@ -829,7 +927,7 @@ static int stellaris_protect(struct flash_bank *bank, int set, int first, int la
 	 * it might want to process those IRQs.
 	 */
 
-	LOG_DEBUG("fmppe 0x%" PRIx32 "",fmppe);
+	LOG_DEBUG("fmppe 0x%" PRIx32 "", fmppe);
 	target_write_u32(target, SCB_BASE | FMPPE, fmppe);
 
 	/* Commit FMPPE */
@@ -845,16 +943,13 @@ static int stellaris_protect(struct flash_bank *bank, int set, int first, int la
 	/* target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_COMT); */
 
 	/* Wait until erase complete */
-	do
-	{
+	do {
 		target_read_u32(target, FLASH_FMC, &flash_fmc);
-	}
-	while (flash_fmc & FMC_COMT);
+	} while (flash_fmc & FMC_COMT);
 
 	/* Check acess violations */
 	target_read_u32(target, FLASH_CRIS, &flash_cris);
-	if (flash_cris & (AMASK))
-	{
+	if (flash_cris & (AMASK)) {
 		LOG_WARNING("Error setting flash page protection,  flash_cris 0x%" PRIx32 "", flash_cris);
 		target_write_u32(target, FLASH_CRIS, 0);
 		return ERROR_FLASH_OPERATION_FAILED;
@@ -865,54 +960,51 @@ static int stellaris_protect(struct flash_bank *bank, int set, int first, int la
 
 /* see contib/loaders/flash/stellaris.s for src */
 
-static const uint8_t stellaris_write_code[] =
-{
-/*
-	Call with :
-	r0 = buffer address
-	r1 = destination address
-	r2 = bytecount (in) - endaddr (work)
+static const uint8_t stellaris_write_code[] = {
+								/* write: */
+	0xDF, 0xF8, 0x40, 0x40,		/* ldr		r4, pFLASH_CTRL_BASE */
+	0xDF, 0xF8, 0x40, 0x50,		/* ldr		r5, FLASHWRITECMD */
+								/* wait_fifo: */
+	0xD0, 0xF8, 0x00, 0x80,		/* ldr		r8, [r0, #0] */
+	0xB8, 0xF1, 0x00, 0x0F,		/* cmp		r8, #0 */
+	0x17, 0xD0,					/* beq		exit */
+	0x47, 0x68,					/* ldr		r7, [r0, #4] */
+	0x47, 0x45,					/* cmp		r7, r8 */
+	0xF7, 0xD0,					/* beq		wait_fifo */
+								/* mainloop: */
+	0x22, 0x60,					/* str		r2, [r4, #0] */
+	0x02, 0xF1, 0x04, 0x02,		/* add		r2, r2, #4 */
+	0x57, 0xF8, 0x04, 0x8B,		/* ldr		r8, [r7], #4 */
+	0xC4, 0xF8, 0x04, 0x80,		/* str		r8, [r4, #4] */
+	0xA5, 0x60,					/* str		r5, [r4, #8] */
+								/* busy: */
+	0xD4, 0xF8, 0x08, 0x80,		/* ldr		r8, [r4, #8] */
+	0x18, 0xF0, 0x01, 0x0F,		/* tst		r8, #1 */
+	0xFA, 0xD1,					/* bne		busy */
+	0x8F, 0x42,					/* cmp		r7, r1 */
+	0x28, 0xBF,					/* it		cs */
+	0x00, 0xF1, 0x08, 0x07,		/* addcs	r7, r0, #8 */
+	0x47, 0x60,					/* str		r7, [r0, #4] */
+	0x01, 0x3B,					/* subs		r3, r3, #1 */
+	0x03, 0xB1,					/* cbz		r3, exit */
+	0xE2, 0xE7,					/* b		wait_fifo */
+								/* exit: */
+	0x00, 0xBE,					/* bkpt		#0 */
 
-	Used registers:
-	r3 = pFLASH_CTRL_BASE
-	r4 = FLASHWRITECMD
-	r5 = #1
-	r6 = bytes written
-	r7 = temp reg
-*/
-	0x07,0x4B,			/* ldr r3,pFLASH_CTRL_BASE */
-	0x08,0x4C,			/* ldr r4,FLASHWRITECMD */
-	0x01,0x25,			/* movs r5, 1 */
-	0x00,0x26,			/* movs r6, #0 */
-/* mainloop: */
-	0x19,0x60,			/* str	r1, [r3, #0] */
-	0x87,0x59,			/* ldr	r7, [r0, r6] */
-	0x5F,0x60,			/* str	r7, [r3, #4] */
-	0x9C,0x60,			/* str	r4, [r3, #8] */
-/* waitloop: */
-	0x9F,0x68,			/* ldr	r7, [r3, #8] */
-	0x2F,0x42,			/* tst	r7, r5 */
-	0xFC,0xD1,			/* bne	waitloop */
-	0x04,0x31,			/* adds	r1, r1, #4 */
-	0x04,0x36,			/* adds	r6, r6, #4 */
-	0x96,0x42,			/* cmp	r6, r2 */
-	0xF4,0xD1,			/* bne	mainloop */
-	0x00,0xBE,     		/* bkpt #0 */
-/* pFLASH_CTRL_BASE: */
-	0x00,0xD0,0x0F,0x40,	/* .word	0x400FD000 */
-/* FLASHWRITECMD: */
-	0x01,0x00,0x42,0xA4	/* .word	0xA4420001 */
+	/* pFLASH_CTRL_BASE: */
+	0x00, 0xD0, 0x0F, 0x40,	/* .word	0x400FD000 */
+	/* FLASHWRITECMD: */
+	0x01, 0x00, 0x42, 0xA4	/* .word	0xA4420001 */
 };
-
 static int stellaris_write_block(struct flash_bank *bank,
-		uint8_t *buffer, uint32_t offset, uint32_t wcount)
+		const uint8_t *buffer, uint32_t offset, uint32_t wcount)
 {
 	struct target *target = bank->target;
 	uint32_t buffer_size = 16384;
 	struct working_area *source;
 	struct working_area *write_algorithm;
 	uint32_t address = bank->base + offset;
-	struct reg_param reg_params[3];
+	struct reg_param reg_params[4];
 	struct armv7m_algorithm armv7m_info;
 	int retval = ERROR_OK;
 
@@ -927,8 +1019,8 @@ static int stellaris_write_block(struct flash_bank *bank,
 			bank, buffer, offset, wcount);
 
 	/* flash write code */
-	if (target_alloc_working_area(target, sizeof(stellaris_write_code), &write_algorithm) != ERROR_OK)
-	{
+	if (target_alloc_working_area(target, sizeof(stellaris_write_code),
+			&write_algorithm) != ERROR_OK) {
 		LOG_DEBUG("no working area for block memory writes");
 		return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
 	};
@@ -938,11 +1030,9 @@ static int stellaris_write_block(struct flash_bank *bank,
 		buffer_size = wcount * 4;
 
 	/* memory buffer */
-	while (target_alloc_working_area_try(target, buffer_size, &source) != ERROR_OK)
-	{
+	while (target_alloc_working_area_try(target, buffer_size, &source) != ERROR_OK) {
 		buffer_size /= 2;
-		if (buffer_size <= buf_min)
-		{
+		if (buffer_size <= buf_min) {
 			target_free_working_area(target, write_algorithm);
 			return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
 		}
@@ -950,51 +1040,32 @@ static int stellaris_write_block(struct flash_bank *bank,
 				target_name(target), (unsigned) buffer_size);
 	};
 
-	retval = target_write_buffer(target, write_algorithm->address,
+	target_write_buffer(target, write_algorithm->address,
 			sizeof(stellaris_write_code),
-			(uint8_t *) stellaris_write_code);
+			stellaris_write_code);
 
 	armv7m_info.common_magic = ARMV7M_COMMON_MAGIC;
-	armv7m_info.core_mode = ARMV7M_MODE_ANY;
+	armv7m_info.core_mode = ARM_MODE_THREAD;
 
 	init_reg_param(&reg_params[0], "r0", 32, PARAM_OUT);
 	init_reg_param(&reg_params[1], "r1", 32, PARAM_OUT);
 	init_reg_param(&reg_params[2], "r2", 32, PARAM_OUT);
+	init_reg_param(&reg_params[3], "r3", 32, PARAM_OUT);
 
-	while (wcount > 0)
-	{
-		uint32_t thisrun_count = (wcount > (buffer_size / 4)) ? (buffer_size / 4) : wcount;
+	buf_set_u32(reg_params[0].value, 0, 32, source->address);
+	buf_set_u32(reg_params[1].value, 0, 32, source->address + source->size);
+	buf_set_u32(reg_params[2].value, 0, 32, address);
+	buf_set_u32(reg_params[3].value, 0, 32, wcount);
 
-		target_write_buffer(target, source->address, thisrun_count * 4, buffer);
+	retval = target_run_flash_async_algorithm(target, buffer, wcount, 4,
+			0, NULL,
+			4, reg_params,
+			source->address, source->size,
+			write_algorithm->address, 0,
+			&armv7m_info);
 
-		buf_set_u32(reg_params[0].value, 0, 32, source->address);
-		buf_set_u32(reg_params[1].value, 0, 32, address);
-		buf_set_u32(reg_params[2].value, 0, 32, 4*thisrun_count);
-		LOG_DEBUG("Algorithm flash write %u words to 0x%" PRIx32
-				", %u remaining",
-				(unsigned) thisrun_count, address,
-				(unsigned) (wcount - thisrun_count));
-		retval = target_run_algorithm(target, 0, NULL, 3, reg_params,
-				write_algorithm->address,
-				0,
-				10000, &armv7m_info);
-		if (retval != ERROR_OK)
-		{
-			LOG_ERROR("error %d executing stellaris "
-					"flash write algorithm",
-					retval);
-			retval = ERROR_FLASH_OPERATION_FAILED;
-			break;
-		}
-
-		buffer += thisrun_count * 4;
-		address += thisrun_count * 4;
-		wcount -= thisrun_count;
-	}
-
-	/* REVISIT we could speed up writing multi-section images by
-	 * not freeing the initialized write_algorithm this way.
-	 */
+	if (retval == ERROR_FLASH_OPERATION_FAILED)
+		LOG_ERROR("error %d executing stellaris flash write algorithm", retval);
 
 	target_free_working_area(target, write_algorithm);
 	target_free_working_area(target, source);
@@ -1002,11 +1073,13 @@ static int stellaris_write_block(struct flash_bank *bank,
 	destroy_reg_param(&reg_params[0]);
 	destroy_reg_param(&reg_params[1]);
 	destroy_reg_param(&reg_params[2]);
+	destroy_reg_param(&reg_params[3]);
 
 	return retval;
 }
 
-static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t offset, uint32_t count)
+static int stellaris_write(struct flash_bank *bank, const uint8_t *buffer,
+		uint32_t offset, uint32_t count)
 {
 	struct stellaris_flash_bank *stellaris_info = bank->driver_priv;
 	struct target *target = bank->target;
@@ -1017,8 +1090,7 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 	uint32_t bytes_written = 0;
 	int retval;
 
-	if (bank->target->state != TARGET_HALTED)
-	{
+	if (bank->target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
@@ -1029,8 +1101,7 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 	if (stellaris_info->did1 == 0)
 		return ERROR_FLASH_BANK_NOT_PROBED;
 
-	if (offset & 0x3)
-	{
+	if (offset & 0x3) {
 		LOG_WARNING("offset size must be word aligned");
 		return ERROR_FLASH_DST_BREAKS_ALIGNMENT;
 	}
@@ -1051,36 +1122,28 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 	 */
 
 	/* multiple words to be programmed? */
-	if (words_remaining > 0)
-	{
+	if (words_remaining > 0) {
 		/* try using a block write */
 		retval = stellaris_write_block(bank, buffer, offset,
 				words_remaining);
-		if (retval != ERROR_OK)
-		{
-			if (retval == ERROR_TARGET_RESOURCE_NOT_AVAILABLE)
-			{
+		if (retval != ERROR_OK) {
+			if (retval == ERROR_TARGET_RESOURCE_NOT_AVAILABLE) {
 				LOG_DEBUG("writing flash word-at-a-time");
-			}
-			else if (retval == ERROR_FLASH_OPERATION_FAILED)
-			{
+			} else if (retval == ERROR_FLASH_OPERATION_FAILED) {
 				/* if an error occured, we examine the reason, and quit */
 				target_read_u32(target, FLASH_CRIS, &flash_cris);
 
 				LOG_ERROR("flash writing failed with CRIS: 0x%" PRIx32 "", flash_cris);
 				return ERROR_FLASH_OPERATION_FAILED;
 			}
-		}
-		else
-		{
+		} else {
 			buffer += words_remaining * 4;
 			address += words_remaining * 4;
 			words_remaining = 0;
 		}
 	}
 
-	while (words_remaining > 0)
-	{
+	while (words_remaining > 0) {
 		if (!(address & 0xff))
 			LOG_DEBUG("0x%" PRIx32 "", address);
 
@@ -1090,8 +1153,7 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 		target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_WRITE);
 		/* LOG_DEBUG("0x%x 0x%x 0x%x",address,buf_get_u32(buffer, 0, 32),FMC_WRKEY | FMC_WRITE); */
 		/* Wait until write complete */
-		do
-		{
+		do {
 			target_read_u32(target, FLASH_FMC, &flash_fmc);
 		} while (flash_fmc & FMC_WRITE);
 
@@ -1100,17 +1162,11 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 		words_remaining--;
 	}
 
-	if (bytes_remaining)
-	{
+	if (bytes_remaining) {
 		uint8_t last_word[4] = {0xff, 0xff, 0xff, 0xff};
-		int i = 0;
 
-		while (bytes_remaining > 0)
-		{
-			last_word[i++] = *(buffer + bytes_written);
-			bytes_remaining--;
-			bytes_written++;
-		}
+		/* copy the last remaining bytes into the write buffer */
+		memcpy(last_word, buffer+bytes_written, bytes_remaining);
 
 		if (!(address & 0xff))
 			LOG_DEBUG("0x%" PRIx32 "", address);
@@ -1121,16 +1177,14 @@ static int stellaris_write(struct flash_bank *bank, uint8_t *buffer, uint32_t of
 		target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_WRITE);
 		/* LOG_DEBUG("0x%x 0x%x 0x%x",address,buf_get_u32(buffer, 0, 32),FMC_WRKEY | FMC_WRITE); */
 		/* Wait until write complete */
-		do
-		{
+		do {
 			target_read_u32(target, FLASH_FMC, &flash_fmc);
 		} while (flash_fmc & FMC_WRITE);
 	}
 
 	/* Check access violations */
 	target_read_u32(target, FLASH_CRIS, &flash_cris);
-	if (flash_cris & (AMASK))
-	{
+	if (flash_cris & (AMASK)) {
 		LOG_DEBUG("flash_cris 0x%" PRIx32 "", flash_cris);
 		return ERROR_FLASH_OPERATION_FAILED;
 	}
@@ -1156,8 +1210,7 @@ static int stellaris_probe(struct flash_bank *bank)
 	if (retval != ERROR_OK)
 		return retval;
 
-	if (bank->sectors)
-	{
+	if (bank->sectors) {
 		free(bank->sectors);
 		bank->sectors = NULL;
 	}
@@ -1166,8 +1219,7 @@ static int stellaris_probe(struct flash_bank *bank)
 	bank->size = 1024 * stellaris_info->num_pages;
 	bank->num_sectors = stellaris_info->num_pages;
 	bank->sectors = calloc(bank->num_sectors, sizeof(struct flash_sector));
-	for (int i = 0; i < bank->num_sectors; i++)
-	{
+	for (int i = 0; i < bank->num_sectors; i++) {
 		bank->sectors[i].offset = i * stellaris_info->pagesize;
 		bank->sectors[i].size = stellaris_info->pagesize;
 		bank->sectors[i].is_erased = -1;
@@ -1186,8 +1238,7 @@ static int stellaris_mass_erase(struct flash_bank *bank)
 	stellaris_info = bank->driver_priv;
 	target = bank->target;
 
-	if (target->state != TARGET_HALTED)
-	{
+	if (target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
@@ -1210,24 +1261,19 @@ static int stellaris_mass_erase(struct flash_bank *bank)
 	target_write_u32(target, FLASH_FMA, 0);
 	target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_MERASE);
 	/* Wait until erase complete */
-	do
-	{
+	do {
 		target_read_u32(target, FLASH_FMC, &flash_fmc);
-	}
-	while (flash_fmc & FMC_MERASE);
+	} while (flash_fmc & FMC_MERASE);
 
 	/* if device has > 128k, then second erase cycle is needed
 	 * this is only valid for older devices, but will not hurt */
-	if (stellaris_info->num_pages * stellaris_info->pagesize > 0x20000)
-	{
+	if (stellaris_info->num_pages * stellaris_info->pagesize > 0x20000) {
 		target_write_u32(target, FLASH_FMA, 0x20000);
 		target_write_u32(target, FLASH_FMC, FMC_WRKEY | FMC_MERASE);
 		/* Wait until erase complete */
-		do
-		{
+		do {
 			target_read_u32(target, FLASH_FMC, &flash_fmc);
-		}
-		while (flash_fmc & FMC_MERASE);
+		} while (flash_fmc & FMC_MERASE);
 	}
 
 	return ERROR_OK;
@@ -1238,30 +1284,21 @@ COMMAND_HANDLER(stellaris_handle_mass_erase_command)
 	int i;
 
 	if (CMD_ARGC < 1)
-	{
-		command_print(CMD_CTX, "stellaris mass_erase <bank>");
-		return ERROR_OK;
-	}
+		return ERROR_COMMAND_SYNTAX_ERROR;
 
 	struct flash_bank *bank;
 	int retval = CALL_COMMAND_HANDLER(flash_command_get_bank, 0, &bank);
 	if (ERROR_OK != retval)
 		return retval;
 
-	if (stellaris_mass_erase(bank) == ERROR_OK)
-	{
+	if (stellaris_mass_erase(bank) == ERROR_OK) {
 		/* set all sectors as erased */
 		for (i = 0; i < bank->num_sectors; i++)
-		{
 			bank->sectors[i].is_erased = 1;
-		}
 
 		command_print(CMD_CTX, "stellaris mass erase complete");
-	}
-	else
-	{
+	} else
 		command_print(CMD_CTX, "stellaris mass erase failed");
-	}
 
 	return ERROR_OK;
 }
@@ -1295,7 +1332,7 @@ COMMAND_HANDLER(stellaris_handle_recover_command)
 		LOG_ERROR("Can't recover Stellaris flash without SRST");
 		return ERROR_FAIL;
 	}
-	jtag_add_reset(0, 1);
+	adapter_assert_reset();
 
 	for (int i = 0; i < 5; i++) {
 		retval = dap_to_swd(bank->target);
@@ -1308,7 +1345,7 @@ COMMAND_HANDLER(stellaris_handle_recover_command)
 	}
 
 	/* de-assert SRST */
-	jtag_add_reset(0, 0);
+	adapter_deassert_reset();
 	retval = jtag_execute_queue();
 
 	/* wait 400+ msec ... OK, "1+ second" is simpler */
@@ -1327,9 +1364,9 @@ done:
 static const struct command_registration stellaris_exec_command_handlers[] = {
 	{
 		.name = "mass_erase",
+		.usage = "<bank>",
 		.handler = stellaris_handle_mass_erase_command,
 		.mode = COMMAND_EXEC,
-		.usage = "bank_id",
 		.help = "erase entire device",
 	},
 	{
@@ -1346,6 +1383,7 @@ static const struct command_registration stellaris_command_handlers[] = {
 		.name = "stellaris",
 		.mode = COMMAND_EXEC,
 		.help = "Stellaris flash command group",
+		.usage = "",
 		.chain = stellaris_exec_command_handlers,
 	},
 	COMMAND_REGISTRATION_DONE
@@ -1361,7 +1399,7 @@ struct flash_driver stellaris_flash = {
 	.read = default_flash_read,
 	.probe = stellaris_probe,
 	.auto_probe = stellaris_probe,
-	.erase_check = default_flash_mem_blank_check,
+	.erase_check = default_flash_blank_check,
 	.protect_check = stellaris_protect_check,
 	.info = get_stellaris_info,
 };
